'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var path = _interopDefault(require('path'));
var majo = _interopDefault(require('majo'));
var match = _interopDefault(require('minimatch'));
var inquirer = _interopDefault(require('inquirer'));
var match$1 = _interopDefault(require('multimatch'));
var isBinaryPath = _interopDefault(require('is-binary-path'));
var exists = _interopDefault(require('path-exists'));

// forked from https://github.com/vuejs/vue-cli/blob/master/lib/eval.js

function evalualte(exp, data) {
  /* eslint-disable no-new-func */
  var fn = new Function('data', `with (data) { return ${exp} }`);
  try {
    return fn(data);
  } catch (err) {
    console.error(`Error when evaluating filter condition: ${exp}`);
  }
}

// forked from https://github.com/vuejs/vue-cli/blob/master/lib/filter.js
function filterFiles(filters) {
  return function (ctx) {
    if (!filters) return;

    var fileList = ctx.fileList;
    var data = ctx.meta.merged;

    Object.keys(filters).forEach(function (glob) {
      fileList.forEach(function (file) {
        if (match(file, glob, { dot: true })) {
          var condition = filters[glob];
          if (!evalualte(condition, data)) {
            ctx.deleteFile(file);
          }
        }
      });
    });
  };
}

function ask(data, prompts) {
  return function (ctx) {
    if (prompts) {
      return inquirer.prompt(prompts).then(function (answers) {
        data = typeof data === 'function' ? data(answers) : data;
        var merged = Object.assign({}, data, answers);

        // prevent from ReferenceErrors
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = prompts[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var prompt = _step.value;

            if (!Object.prototype.hasOwnProperty.call(merged, prompt.name)) {
              merged[prompt.name] = undefined;
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        ctx.meta = { data, answers, merged };
      });
    }

    if (data) {
      ctx.meta = { data, merged: data };
    } else {
      ctx.meta = { merged: {} };
    }
  };
}

var useTemplate = (function () {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      skipInterpolation = _ref.skipInterpolation,
      _ref$template = _ref.template,
      template = _ref$template === undefined ? require('jstransformer-ejs') : _ref$template,
      _ref$templateOptions = _ref.templateOptions,
      templateOptions = _ref$templateOptions === undefined ? {} : _ref$templateOptions;

  return function (ctx) {
    templateOptions = typeof templateOptions === 'function' ? templateOptions(ctx.meta) : templateOptions;

    var fileList = ctx.fileList;
    var matchedFile = void 0;
    if (skipInterpolation) {
      if (typeof skipInterpolation === 'function') {
        matchedFile = skipInterpolation;
      } else {
        var matches = match$1(fileList, skipInterpolation);
        matchedFile = function matchedFile(file) {
          return matches.indexOf(file) !== -1;
        };
      }
    }

    return Promise.all(fileList.map(function (relative) {
      return run(relative);
    }));

    function run(file) {
      var content = ctx.fileContents(file);

      var shouldSkip = matchedFile && matchedFile(file, content);

      // we skip unmathed files (by multimatch or your own function)
      // and binary files
      if (shouldSkip || isBinaryPath(file)) {
        return;
      }

      var res = require('jstransformer')(template).render(content, templateOptions, ctx.meta.merged);
      ctx.writeContents(file, res.body);
    }
  };
});

var skip = function (skipExisting, destPath) {
  return function (ctx) {
    return Promise.all(ctx.fileList.map(function (name) {
      var location = path.join(destPath, name);
      return exists(location).then(function (yes) {
        if (yes) {
          ctx.deleteFile(name);
          if (typeof skipExisting === 'function') {
            skipExisting(location, name);
          }
        }
      });
    }));
  };
};

var moveFiles = function (move) {
  return function (ctx) {
    if (!move) return;

    for (var pattern in move) {
      var matches = match.match(ctx.fileList, pattern);
      if (matches.length > 0) {
        var file = ctx.file(matches[0]);
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = matches[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var match$$1 = _step.value;

            ctx.deleteFile(match$$1);
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        var newName = move[pattern];
        ctx.createFile(newName, file);
      }
    }
  };
};

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function kopy(src, dest) {
  var _ref = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {},
      _ref$cwd = _ref.cwd,
      cwd = _ref$cwd === undefined ? process.cwd() : _ref$cwd,
      _ref$clean = _ref.clean,
      clean = _ref$clean === undefined ? false : _ref$clean,
      data = _ref.data,
      prompts = _ref.prompts,
      _ref$disableInterpola = _ref.disableInterpolation,
      disableInterpolation = _ref$disableInterpola === undefined ? false : _ref$disableInterpola,
      skipInterpolation = _ref.skipInterpolation,
      template = _ref.template,
      templateOptions = _ref.templateOptions,
      filters = _ref.filters,
      skipExisting = _ref.skipExisting,
      move = _ref.move,
      _ref$write = _ref.write,
      write = _ref$write === undefined ? true : _ref$write;

  var destPath = path.resolve(cwd, dest);
  var base = path.resolve(cwd, src);

  var done = function done(stream) {
    return _extends({
      files: stream.files,
      fileList: stream.fileList
    }, stream.meta);
  };

  var stream = majo();

  stream.source('**', { cwd: base }).filter(function (file) {
    return !/\.DS_Store$/.test(file);
  }).use(ask(data, prompts)).use(filterFiles(filters)).use(moveFiles(move));

  if (!disableInterpolation) {
    stream.use(useTemplate({
      skipInterpolation,
      template,
      templateOptions
    }));
  }

  if (skipExisting) {
    stream.use(skip(skipExisting, destPath));
  }

  if (write === false) {
    return stream.process().then(function () {
      return done(stream);
    });
  }

  return stream.dest(destPath, { clean }).then(function () {
    return done(stream);
  });
}

module.exports = kopy;
